<?php

/**
 * Crypto-Processing Library for OpenCart
 */
class Cprocessing
{

    /** @var int $version */
    public $version = '1.0.0';

    /** @var Registry $registry */
    private $registry;

    /** @var Log $logger */
    public $logger;

    /** @var string $cprocessing_websocket_url */
    public $cprocessing_websocket_url;

    /**
     * Crypto-Processing Library constructor
     * @param Registry $registry
     */
    public function __construct($registry)
    {
        $this->registry = $registry;

        // Setup encryption
        $fingerprint = substr(sha1(sha1(__DIR__)), 0, 24);
        $this->encryption = new Encryption($fingerprint);

        // Setup logging
        $this->logger = new Log('cprocessing.log');

        $cprocessing_base_url = 'https://external-api.bcon.global/api/v1';

        //General
        $this->cprocessing_new_address_url = $cprocessing_base_url . '/address'; // Post
        $this->cprocessing_callback_url = $cprocessing_base_url . '/stores/get_store'; // GET
        $this->cprocessing_set_callback = $cprocessing_base_url . '/stores/set_callback'; // GET

        //BTC
        $this->cprocessing_price_url_btc = $cprocessing_base_url . '/currencies/btc'; // GET

        //BNB
        $this->cprocessing_price_url_bnb = $cprocessing_base_url . '/currencies/bnb'; // GET

        $this->setting('debug', 1);
    }

    /**
     * Magic getter for Registry items
     *
     * Allows use of $this->db instead of $this->registry->get('db') for example
     *
     * @return mixed
     */
    public function __get($name)
    {
        return $this->registry->get($name);
    }

    public function getPrice($crypto)
    {
        //Getting price
        $options = array('http' => array('method' => 'GET'));
        $context = stream_context_create($options);
        $currency_code = $this->config->get('config_currency');
        if($crypto == 'btc') {
            $contents = file_get_contents($this->cprocessing_price_url_btc . '?currency=' . $currency_code);
        } else {
            $contents = file_get_contents($this->cprocessing_price_url_bnb . '?currency=' . $currency_code);
        }
        $priceObj = json_decode($contents);
        return $priceObj->data->price;
    }

    /**
     * Logs with an arbitrary level.
     * @param string $level The type of log.
     *                        Should be 'error', 'warn', 'info', 'debug', 'trace'
     *                        In normal mode, 'error' and 'warn' are logged
     *                        In debug mode, all are logged
     * @param string $message The message of the log
     * @param int $depth How deep to go to find the calling function
     * @return void
     */
    public function log($level, $message, $depth = 0)
    {
        $level = strtoupper($level);
        $prefix = '[' . $level . ']';

        if ($this->setting('debug') === '1') {
            $depth += 1;
            $prefix .= '{';
            $backtrace = debug_backtrace();
            if (isset($backtrace[$depth]['class'])) {
                $class = preg_replace('/[a-z]/', '', $backtrace[$depth]['class']);
                $prefix .= $class . $backtrace[$depth]['type'];
            }
            if (isset($backtrace[$depth]['function'])) {
                $prefix .= $backtrace[$depth]['function'];
            }
            $prefix .= '}';
        }

        if ('ERROR' === $level || 'WARN' === $level || $this->setting('debug') === '1') {
            $this->logger->write($prefix . ' ' . $message);
        }
    }

    public function genAddress($crypto)
    {
        // Set secret
        $url = $this->cprocessing_new_address_url;

        $body = [
            'callback_url' => $this->setting('callback_url')
        ];

        $headers = [
            'Authorization: Bearer ' . $this->setting('api_key_' . $crypto),
            'Accept: application/json'
        ];

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_TIMEOUT, 60);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $body);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        $data = curl_exec($ch);
        $httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);

        $responseObj = json_decode($data);

        if (!isset($responseObj)) {
            return '';
        }

        return $responseObj->data;
    }

    /**
     * Constructs some helpful diagnostic info.
     * @return string
     */
    public function getServerInfo()
    {
        $gmp = extension_loaded('gmp') ? 'enabled' : 'missing';
        $bcmath = extension_loaded('bcmath') ? 'enabled' : 'missing';
        $info = "<pre><strong>Server Information:</strong>\n" .
            "PHP: " . phpversion() . "\n" .
            "PHP-GMP: " . $gmp . "\n" .
            "PHP-BCMATH: " . $bcmath . "\n" .
            "OpenCart: " . VERSION . "\n" .
            "Crypto-Processing Plugin: " . $this->version . "\n" .
            "Crypto-Processing Lib: v1.0.0\n";
        return $info;
    }

    /**
     * Better setting method for new Cprocessing settings
     *
     * Automatically persists to database on set and combines getting and setting into one method
     * Assumes cprocessing_ prefix
     *
     * @param string $key Setting key
     * @param string $value Setting value if setting the value
     * @return string|null|void Setting value, or void if setting the value
     */
    public function setting($key, $value = null)
    {
        // Normalize key
        $code = 'payment_cprocessing';
        $key = $code . '_' . $key;

        // Set the setting
        if (func_num_args() === 2) {
            if (!is_array($value)) {
                $this->db->query("UPDATE " . DB_PREFIX . "setting SET `value` = '" . $this->db->escape($value) . "', serialized = '0' WHERE `code` = '" . $code . "' AND `key` = '" . $this->db->escape($key) . "' AND store_id = '0'");
            } else {
                $this->db->query("UPDATE " . DB_PREFIX . "setting SET `value` = '" . $this->db->escape(serialize($value)) . "', serialized = '1' code `group` = 'cprocessing' AND `key` = '" . $this->db->escape($key) . "' AND store_id = '0'");
            }
            return $this->config->set($key, $value);
        }

        // Get the setting
        return $this->config->get($key);
    }

    private function getStore($crypto = 'btc') {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $this->cprocessing_callback_url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'GET');
        curl_setopt($ch, CURLOPT_TIMEOUT, 60);
        curl_setopt($ch, CURLOPT_POSTFIELDS, "");
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            'Authorization: Bearer ' . $this->setting('api_key_' . $crypto),
            'Content-type: application/x-www-form-urlencoded'
        ));

        $data = curl_exec($ch);
        $httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);

        $responseObj = json_decode($data);
        if (!isset($responseObj) || $httpcode == '401') {
            return '';
        }

        return [
            'address' => $responseObj->data->store->xpub
        ];
    }

    public function setCallbackUrl() {
        $currencies = [];

        if($this->config->get('payment_cprocessing_btc_on')) $currencies[] = 'btc';
        if($this->config->get('payment_cprocessing_bnb_on')) $currencies[] = 'bnb';

        foreach ($currencies as $currency) {
            $xpub = self::getStore($currency);

            $body = [
                'callback_url' => $this->setting('callback_url'),
                'address' => isset($xpub['address']) ? $xpub['address'] : ''
            ];

            $headers = [
                'Authorization: Bearer ' . $this->setting('api_key_' . $currency),
                'Accept: application/json'
            ];

            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $this->cprocessing_set_callback);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_TIMEOUT, 60);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $body);
            curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

            $data = curl_exec($ch);
            $httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
            curl_close($ch);
        }
    }
}
